// MiniScroll_AIO_ESP8266
// CLClab306 5/30/2023
// https://www.instructables.com/Programmable-Sticky-Note-Desktop-Scrolling-LED-Dis/

// Edit this file
#include "config.h"

// Download these libraries
#include "TimeLib.h"
#include <Timezone.h>   // https://github.com/JChristensen/Timezone
#include <MD_Parola.h>
#include <MD_MAX72xx.h>
#include <SPI.h>

// Define the number of devices we have in the chain and the hardware interface
#define HARDWARE_TYPE MD_MAX72XX::FC16_HW
#define MAX_DEVICES 4
#define CLK_PIN   D5          // GPIO14
#define DATA_PIN  D7          // GPIO13
#define CS_PIN    D8          // GPIO15

// HARDWARE SPI
MD_Parola P = MD_Parola(HARDWARE_TYPE, CS_PIN, MAX_DEVICES);

uint8_t scrollSpeed = 25;                     // default frame delay value (lower = faster)
textEffect_t scrollEffect = PA_SCROLL_LEFT;
textPosition_t scrollAlign = PA_LEFT;
uint16_t scrollPause = 1000;                  // pause at end of each scroll in milliseconds
int counter = 0;

// Message buffers
#define	BUF_SIZE	75
char curMessage[BUF_SIZE] = {"Connected to Adafruit IO.             "};
char newMessage[BUF_SIZE] = {""};
bool newMessageAvailable = false;

// US Pacific Time Zone (Seattle, Portland); >>> EDIT FOR YOUR TIMEZONE <<<
// see https://github.com/JChristensen/Timezone for more information
// also edit line 219 for your timezone
TimeChangeRule usPDT = {"PDT", Second, Sun, Mar, 2, -420};    // Daylight time = UTC - 7 hours
TimeChangeRule usPST = {"PST", First, Sun, Nov, 2, -480};     // Standard time = UTC - 8 hours
Timezone usPacific(usPDT, usPST);

String msg = "";
String scroll0 = "";
String scroll1 = "Message 1";
String scroll2 = "Message 2";
String scroll3 = "Message 3";
int scrollSel = 0;
int onFlg = 0;
int onTime = 0001;        //24-h times
int offTime = 2359;
int current24h = 0;
int last24h = 0;

String dayNames[] = {"ERROR","Sun","Mon","Tue","Wed","Thu","Fri","Sat"};
String timeStr = "99:99";
String dateStr = "99/99/2022";
String digit1 = "X";

// set up the AIO data feeds
AdafruitIO_Feed *on = io.feed("ontime");
AdafruitIO_Feed *off = io.feed("offtime");
AdafruitIO_Feed *sel = io.feed("select");
AdafruitIO_Feed *s1 = io.feed("scroll1");
AdafruitIO_Feed *s2 = io.feed("scroll2");
AdafruitIO_Feed *s3 = io.feed("scroll3");
AdafruitIO_Feed *spd = io.feed("speed");
AdafruitIO_Time *seconds = io.time(AIO_TIME_SECONDS);

void setup()
  {
    P.begin();
    Serial.begin(115200);
    while (!Serial) 
      {
        delay(10);
      }
    delay(500);
    Serial.println(); Serial.println(); 
    Serial.println("Scroll time or text feed from Adafruit IO");  
    Serial.println("Connecting to Adafruit IO");
    // start connection to io.adafruit.com
    io.connect();
    
    // attach message handler for each feed
    on->onMessage(handleON);
    off->onMessage(handleOFF);
    sel->onMessage(handleSel);
    s1->onMessage(handleS1);
    s2->onMessage(handleS2);
    s3->onMessage(handleS3);
    spd->onMessage(handleSpd);
    seconds->onMessage(handleSecs);

    // wait for an AIO connection
    while(io.status() < AIO_CONNECTED) 
      {
        Serial.print(".");  delay(500);
      }
    // AIO connected
    Serial.println();
    Serial.println(io.statusText());
    io.run();
    on->get();        //get last recorded values
    off->get();
    sel->get();                  
    s1->get();
    s2->get();
    s3->get();
    spd->get();

    // Initiate the scrolling display
    P.displayText(curMessage, scrollAlign, scrollSpeed, scrollPause, scrollEffect, scrollEffect);
    P.displayReset();
    
    io.run();
    onFlg = 1;      //force display of starting message
    delay(1000);
  }

void loop()
  {
    if (!onFlg) 
      {
        io.run();           // keep connection to AIO active
        return;             // Do not display if onFlg is not set
      }
    if (P.displayAnimate())
      {
        io.run();                         // keep connection to AIO active
        if (newMessageAvailable)
          {
            switch(scrollSel)
              {
                case 0:                   // Scrolls time, day & date
                  msg = scroll0;
                  break;
                case 1:                   // Scrolls text stored in scroll1
                  msg = scroll1;
                  break;
                case 2:                   // Scrolls text stored in scroll2
                  msg = scroll2;           
                  break;
                case 3:                   // Scrolls text stored in scroll3
                  msg = scroll3;           
                  break;
                default:                  //default scroll = time/day/date
                  msg = scroll0;;           
                  break;
              }
            Serial.println("new message");
            msg.toCharArray(newMessage,BUF_SIZE); 
            strcpy(curMessage, newMessage);
            newMessageAvailable = false;
          }
        P.displayReset();
      }     
  }

void handleON(AdafruitIO_Data *data)
  {
    onTime = data->toInt();
    Serial.print("ON Time: ");  Serial.println(onTime);
    newMessageAvailable = true;
  }

void handleOFF(AdafruitIO_Data *data)
  {
    offTime = data->toInt();
    Serial.print("OFF Time: ");  Serial.println(offTime);
    newMessageAvailable = true;
  }

void handleSel(AdafruitIO_Data *data)
  {
    scrollSel = data->toInt();
    Serial.print("Scroll Select: ");  Serial.println(scrollSel);
    newMessageAvailable = true;
  }
  
void handleS1(AdafruitIO_Data *data)
  {
    scroll1 = data->toString();
    Serial.print("Scroll1 Feed: "); Serial.println(scroll1);
    newMessageAvailable = true;
  }

void handleS2(AdafruitIO_Data *data)
  {
    scroll2 = data->toString();
    Serial.print("Scroll2 Feed: "); Serial.println(scroll2);
    newMessageAvailable = true;
  }

void handleS3(AdafruitIO_Data *data)
  {
    scroll3 = data->toString();
    Serial.print("Scroll3 Feed: "); Serial.println(scroll3);
    newMessageAvailable = true;
  }

void handleSpd(AdafruitIO_Data *data)
  {
    scrollSpeed = data->toInt();
    P.displayText(curMessage, scrollAlign, scrollSpeed, scrollPause, scrollEffect, scrollEffect);
    Serial.print("Scroll Speed Feed: "); Serial.println(scrollSpeed);
    newMessageAvailable = true;
  }

// message handler for the seconds feed
void handleSecs(char *data, uint16_t len)
  {
    //Serial.print("Seconds Feed: ");
    //Serial.println(data);
    String uTime = String(data);
    unsigned long long utc = uTime.toInt();           
    // EDIT NEXT LINE FOR YOUR TIMEZONE
    time_t t = usPacific.toLocal(utc);
    setTime(t);
    //NB: onTime must be less than offTime (on same day)
    current24h = hour(t) * 100 + minute(t);         //convert current h/min to 24-h time integer
    if(current24h >= onTime && current24h < offTime)  onFlg = 1;  //compare to ON and OFF times
    else onFlg = 0;                                               //and change onFlg if needed
    int hour2 = hour(t);
    String XM = "am";
    if (hour(t) == 12)  XM = "pm"; 
    if (hour(t) > 12)
      {
        hour2 = hour(t)-12;
        XM = "pm";
        if(hour2 == 12) XM = "am";
      }
    digit1 = "";
    if ((String(minute(t))).length() == 1) digit1 = "0";
    timeStr = String(hour2) + ":" + digit1 + String(minute(t)) + " " + XM;
    dateStr = String(dayNames[weekday(t)]) + " " + String(month(t)) + "/" + String(day(t)) + "/" + String(year(t)-2000);
    scroll0 = timeStr + "   " + dateStr + "              "; 
    if (last24h != current24h)
      {
        newMessageAvailable = true;
        Serial.print("Current 24h: "); Serial.println(current24h);
      }
    last24h = current24h;
  }